"""MLB Odds Scraper for retrieving odds data from OddsPortal."""

import pandas as pd
import datetime
from selenium import webdriver
from stealthenium import stealth
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from concurrent.futures import ThreadPoolExecutor
import time
import statsapi

def scrape_page(year, page_num):
    """
    Scrape a single page of MLB odds data.
    
    Args:
        year (int): Year to scrape data for
        page_num (int): Page number to scrape
        
    Returns:
        list: List of dictionaries containing game data
    """
    options = webdriver.ChromeOptions()
    options.add_argument("--headless")
    options.add_argument("start-maximized") 
    options.add_argument("--disable-blink-features=AutomationControlled")
    options.add_experimental_option("excludeSwitches", ["enable-automation"])
    options.add_experimental_option('useAutomationExtension', False)

    driver = webdriver.Chrome(options=options)
    
    stealth(driver,
        user_agent='Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/123.0.0.0 Safari/537.36',
        languages=["en-US", "en"],
        vendor="Google Inc.",
        platform="Win32",
        webgl_vendor="Intel Inc.",
        renderer="Intel Iris OpenGL Engine",
        fix_hairline=True,
    )
    
    games = []
    
    # For current year, use URL without year suffix
    current_year = datetime.datetime.now().year
    if year == current_year:
        url = f"https://www.oddsportal.com/baseball/usa/mlb/results/#/page/{page_num}/"
    else:
        url = f"https://www.oddsportal.com/baseball/usa/mlb-{year}/results/#/page/{page_num}/"
        
    print(f"Scraping {url}")
    
    try:
        driver.get(url)
        
        # Handle cookie consent if it appears
        try:
            WebDriverWait(driver, 10).until(
                EC.element_to_be_clickable((By.ID, "onetrust-reject-all-handler"))
            ).click()
            print("Clicked cookie reject button")
        except:
            try:
                WebDriverWait(driver, 5).until(
                    EC.element_to_be_clickable((By.XPATH, "//button[contains(text(), 'Reject')]"))
                ).click()
                print("Clicked alternative cookie reject button")
            except:
                print("No cookie consent dialog found or couldn't be clicked")
        
        # Wait for page to load
        time.sleep(5)
        
        # Try different selectors for event rows
        selectors = [
            "div.eventRow",
            "div[data-v-b8d70024].eventRow",
            "div[data-v-b8d70024][id][set]",
            "div.event__match",
            "div.table-main__tr",
            "div.result-table__row"
        ]
        
        events = []
        for selector in selectors:
            try:
                print(f"Trying selector: {selector}")
                WebDriverWait(driver, 5).until(
                    EC.presence_of_element_located((By.CSS_SELECTOR, selector))
                )
                events = driver.find_elements(By.CSS_SELECTOR, selector)
                if events:
                    print(f"Found {len(events)} events with selector: {selector}")
                    break
            except Exception as e:
                print(f"Selector {selector} failed: {str(e)}")
        
        # If no events found with CSS selectors, try XPath
        if not events:
            try:
                print("Trying XPath selector")
                events = driver.find_elements(By.XPATH, "//div[contains(@class, 'event') and .//div[contains(@class, 'participant')]]")
                print(f"Found {len(events)} events with XPath selector")
            except Exception as e:
                print(f"XPath selector failed: {str(e)}")
        
        # Scroll to load all content
        driver.execute_script("window.scrollTo(0, document.body.scrollHeight);")
        time.sleep(2)
        
        print(f"Found {len(events)} events on page {page_num}")
        current_date = None
        
        # Find initial date
        date_selectors = [
            ".text-black-main",
            ".event__time",
            ".table-main__date",
            ".date"
        ]
        
        for event in events:
            for date_selector in date_selectors:
                try:
                    date_elements = event.find_elements(By.CSS_SELECTOR, date_selector)
                    for date_element in date_elements:
                        date_text = date_element.text
                        if len(date_text.split()) >= 3 and any(yr in date_text for yr in [str(year)]) and "Baseball" not in date_text:
                            if " - " in date_text:
                                current_date = date_text.split(' - ')[0].strip()
                            else:
                                current_date = date_text.strip()
                            break
                    if current_date:
                        break
                except:
                    continue
            if current_date:
                break
                
        # Process events
        for event in events:
            try:
                # Try to get date for this event
                for date_selector in date_selectors:
                    try:
                        date_elements = event.find_elements(By.CSS_SELECTOR, date_selector)
                        for date_element in date_elements:
                            date_text = date_element.text
                            if len(date_text.split()) >= 3 and any(yr in date_text for yr in [str(year)]) and "Baseball" not in date_text:
                                if " - " in date_text:
                                    current_date = date_text.split(' - ')[0].strip()
                                else:
                                    current_date = date_text.strip()
                                break
                    except:
                        continue
                
                # Try different selectors for team names
                team_selectors = [
                    ".participant-name",
                    "p.participant-name",
                    ".event__participant",
                    ".table-main__participant",
                    ".participant"
                ]
                
                teams = []
                for team_selector in team_selectors:
                    try:
                        teams = event.find_elements(By.CSS_SELECTOR, team_selector)
                        if len(teams) >= 2:
                            break
                    except:
                        continue
                
                # Try different selectors for scores
                score_selectors = [
                    "div.text-gray-dark div.font-bold",
                    ".font-bold",
                    ".event__score",
                    ".table-main__score",
                    ".score"
                ]
                
                home_score = ''
                away_score = ''
                for score_selector in score_selectors:
                    try:
                        score_section = event.find_element(By.CSS_SELECTOR, score_selector)
                        score_text = score_section.text
                        if '–' in score_text:
                            scores = score_text.split('–')
                            home_score = scores[0].strip()
                            away_score = scores[1].strip()
                            break
                        elif ':' in score_text and score_text[0].isdigit():
                            scores = score_text.split(':')
                            home_score = scores[0].strip()
                            away_score = scores[1].strip()
                            break
                    except:
                        try:
                            score_elements = event.find_elements(By.CSS_SELECTOR, score_selector)
                            if len(score_elements) >= 2:
                                home_score = score_elements[0].text.strip()
                                away_score = score_elements[1].text.strip()
                                break
                        except:
                            continue
                    
                if len(teams) < 2:
                    continue

                # Try different selectors for time
                time_selectors = [
                    ".//p[contains(text(), ':')]",
                    ".//span[contains(text(), ':')]",
                    ".//div[contains(text(), ':') and string-length(.) <= 8]"
                ]
                
                game_time = None
                for time_selector in time_selectors:
                    try:
                        time_element = event.find_element(By.XPATH, time_selector)
                        time_text = time_element.text
                        if ':' in time_text and len(time_text) <= 8:
                            game_time = time_text
                            break
                    except:
                        continue
                
                if not game_time:
                    continue

                home_team = teams[0].text
                away_team = teams[1].text
                    
                # Try different selectors for odds
                odds_selectors = [
                    "[data-testid='add-to-coupon-button'] .height-content",
                    ".gradient-green-added-border",
                    ".event__odd",
                    ".table-main__odd",
                    ".odd"
                ]
                
                odds_elements = []
                for odds_selector in odds_selectors:
                    try:
                        odds_elements = event.find_elements(By.CSS_SELECTOR, odds_selector)
                        if len(odds_elements) >= 2:
                            break
                    except:
                        continue

                if len(odds_elements) >= 2:
                    home_odds = odds_elements[0].text
                    away_odds = odds_elements[1].text

                    if all([current_date, game_time, away_team, home_team, away_odds, home_odds]):
                        game_data = {
                            'date': current_date,
                            'time': game_time,
                            'away_team': away_team,
                            'home_team': home_team,
                            'away_odds': away_odds,
                            'home_odds': home_odds,
                            'away_score': away_score,
                            'home_score': home_score
                        }
                        games.append(game_data)
                
            except Exception as e:
                print(f"Error processing event: {e}")
                continue
        
    except Exception as e:
        print(f"Error scraping page {page_num} for year {year}: {e}")
    finally:
        driver.quit()
        
    return games

def scrape_oddsportal_mlb(year, max_pages=None):
    """
    Scrape MLB odds data for a specific year.
    
    Args:
        year (int): Year to scrape data for
        max_pages (int, optional): Maximum number of pages to scrape
        
    Returns:
        list: List of dictionaries containing game data
    """
    # Get total pages
    options = webdriver.ChromeOptions()
    options.add_argument("--headless")
    driver = webdriver.Chrome(options=options)
    
    total_pages = 1
    try:
        # For current year, use URL without year suffix
        current_year = datetime.datetime.now().year
        if year == current_year:
            url = f"https://www.oddsportal.com/baseball/usa/mlb/results/#/page/1/"
        else:
            url = f"https://www.oddsportal.com/baseball/usa/mlb-{year}/results/#/page/1/"
            
        print(f"Getting pagination info from {url}")
        driver.get(url)
        
        # Handle cookie consent if it appears
        try:
            WebDriverWait(driver, 10).until(
                EC.element_to_be_clickable((By.ID, "onetrust-reject-all-handler"))
            ).click()
        except:
            try:
                WebDriverWait(driver, 5).until(
                    EC.element_to_be_clickable((By.XPATH, "//button[contains(text(), 'Reject')]"))
                ).click()
            except:
                print("No cookie consent dialog found or couldn't be clicked")
        
        # Wait for page to load
        time.sleep(5)
        
        # Try different selectors for pagination
        pagination_selectors = [
            "div.pagination",
            "ul.pagination",
            ".pagination-container",
            ".pagination-wrapper"
        ]
        
        for selector in pagination_selectors:
            try:
                pagination = WebDriverWait(driver, 10).until(
                    EC.presence_of_element_located((By.CSS_SELECTOR, selector))
                )
                page_links = pagination.find_elements(By.CSS_SELECTOR, "a.pagination-link, a[data-number]")
                
                if page_links:
                    page_numbers = []
                    for link in page_links:
                        data_number = link.get_attribute('data-number')
                        if data_number and data_number.isdigit():
                            page_numbers.append(int(data_number))
                        elif link.text and link.text.isdigit():
                            page_numbers.append(int(link.text))
                    
                    if page_numbers:
                        total_pages = max(page_numbers)
                        break
            except:
                continue
        
        # If pagination not found, check if there's any indication of multiple pages
        if total_pages == 1:
            try:
                next_page = driver.find_element(By.XPATH, "//a[contains(text(), 'Next') or contains(@class, 'next')]")
                if next_page:
                    total_pages = 2  # At least 2 pages
            except:
                pass
    except Exception as e:
        print(f"Error determining total pages for year {year}: {e}")
    finally:
        driver.quit()
    
    if max_pages:
        total_pages = min(max_pages, total_pages)
    
    print(f"Will scrape {total_pages} pages")
    
    # Scrape pages concurrently
    all_games = []
    with ThreadPoolExecutor(max_workers=5) as executor:
        futures = [executor.submit(scrape_page, year, page) for page in range(1, total_pages + 1)]
        for future in futures:
            all_games.extend(future.result())
            
    print(f"Total games collected: {len(all_games)}")
    return all_games

def scrape_oddsportal_mlb_years(start_year=2006, end_year=2025, max_pages=None):
    """
    Scrape MLB odds data for multiple years.
    
    Args:
        start_year (int): First year to scrape (inclusive)
        end_year (int): Last year to scrape (exclusive)
        max_pages (int, optional): Maximum number of pages to scrape per year
        
    Returns:
        pd.DataFrame: DataFrame containing all scraped games
    """
    all_games = []
    for year in range(start_year, end_year):
        print(f"\nScraping year {year}...")
        games = scrape_oddsportal_mlb(year, max_pages)
        
        if games:
            all_games.extend(games)
        else:
            print(f"No games found for year {year}")
        
    return pd.DataFrame(all_games) if all_games else pd.DataFrame()

def process_game_data(df):
    """
    Process raw MLB game data by cleaning and transforming fields.
    
    Args:
        df (pd.DataFrame): Raw dataframe containing MLB game data
        
    Returns:
        pd.DataFrame: Processed dataframe
    """
    if df.empty:
        print("DataFrame is empty, not processing")
        return df
    
    processed_df = df.copy()
    
    # Convert date to datetime
    try:
        processed_df['game_date'] = pd.to_datetime(processed_df['date'])
        processed_df['game_datetime'] = pd.to_datetime(processed_df['date'] + ' ' + processed_df['time'])
        
        # Try to localize to Eastern time and convert to UTC
        try:
            processed_df['game_datetime'] = processed_df['game_datetime'].dt.tz_localize('US/Eastern').dt.tz_convert('UTC')
        except:
            print("Could not convert datetime to UTC, using naive datetime")
    except Exception as e:
        print(f"Error converting dates: {e}")
        processed_df['game_date'] = processed_df['date']
        processed_df['game_datetime'] = processed_df['date'] + ' ' + processed_df['time']
    
    # Drop rows containing All Star Game matchups
    processed_df = processed_df[~(processed_df['away_team'].isin(['American League', 'National League']) | 
                                processed_df['home_team'].isin(['American League', 'National League']))]
    
    # Clean team names
    processed_df['home_team_abbr'] = processed_df['home_team'].str.replace('St.Louis', 'St. Louis')
    processed_df['away_team_abbr'] = processed_df['away_team'].str.replace('St.Louis', 'St. Louis')
    
    # Get unique team names
    teams = pd.unique(processed_df['away_team_abbr'].tolist() + processed_df['home_team_abbr'].tolist())

    # Create mapping of team names to IDs
    team_mapping = {}
    for team in teams:
        try:
            team_info = statsapi.lookup_team(team)
            if team_info:
                team_mapping[team] = team_info[0]['id']
            else:
                team_mapping[team] = None
        except Exception as e:
            print(f"Error looking up team {team}: {e}")
            team_mapping[team] = None

    # Map team names to IDs in processed_df
    processed_df['home_team_id'] = processed_df['home_team_abbr'].map(team_mapping)
    processed_df['away_team_id'] = processed_df['away_team_abbr'].map(team_mapping)
    
    # Convert odds columns - replace + with empty string and convert to numeric
    try:
        processed_df['away_odds'] = pd.to_numeric(processed_df['away_odds'].str.replace('+', ''), errors='coerce')
        processed_df['home_odds'] = pd.to_numeric(processed_df['home_odds'].str.replace('+', ''), errors='coerce')
    except Exception as e:
        print(f"Error converting odds: {e}")

    # Convert score columns to numeric
    try:
        processed_df['away_score'] = pd.to_numeric(processed_df['away_score'], errors='coerce')
        processed_df['home_score'] = pd.to_numeric(processed_df['home_score'], errors='coerce')
    except Exception as e:
        print(f"Error converting scores: {e}")
    
    # Drop rows with missing scores if they exist
    if 'away_score' in processed_df.columns and 'home_score' in processed_df.columns:
        processed_df = processed_df.dropna(subset=['home_score','away_score'])
    
    # Select final columns
    final_columns = ['game_date', 'game_datetime', 'home_team_id', 'away_team_id', 
                     'home_odds', 'away_odds', 'home_score', 'away_score', 
                     'home_team_abbr', 'away_team_abbr']
    
    # Only keep columns that exist
    final_columns = [col for col in final_columns if col in processed_df.columns]
    processed_df = processed_df[final_columns]
    
    return processed_df 